/*----------------------------------------------------------------------------
 Copyright:      
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        30.12.2013
 Description:  

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.   
------------------------------------------------------------------------------*/
#define F_CPU 12000000
#define MAX_WS2811_LEDS 16

#include <util/delay.h>
#include <avr/io.h>
#include <math.h>
#include <avr/interrupt.h>

#include "light_ws2812.h"

struct CRGB { uint8_t g; uint8_t r; uint8_t b; };
struct CRGB led[255];


//############################################################################
//HSV to RGB 8Bit
//Farbkreis h = 0 bis 360 (Farbwert)
//          s = 0 bis 100 (Dunkelstufe)
//          v = 0 bis 100 (Farbsttigung)
//Rckgabewert r,g,b als Pointer
void hsv_to_rgb (unsigned int h,unsigned char s,unsigned char v,
					unsigned char* r, unsigned char* g, unsigned char* b)
{		
		unsigned char max_v;
		unsigned char diff;
	
		//Winkel im Farbkeis 0 - 360 in 1 Grad Schritten
		//h = (englisch hue) Farbwert
		//1 Grad Schrittweite, 4.25 Steigung pro Schritt bei 60 Grad
		if(h<61){
			*r = 255;
			*b = 0;
			*g = 4.25 * h;
		}else if(h < 121){
			*g = 255;
			*b = 0;
			*r = 255 - (4.25 * (h-60));
		}else if(h < 181){
			*r = 0;
			*g = 255;
			*b = 4.25 * (h-120);
		}else if(h < 241){
			*r = 0;
			*b = 255;
			*g = 255 - (4.25 * (h-180));
		}else if(h < 301){
			*g = 0;
			*b = 255;
			*r = 4.25 * (h-240);
		}else if(h< 360){
			*r = 255;
			*g = 0;
			*b = 255 - (4.25 * (h-300));
		}	
		
		//Berechnung der Dunkelstufe
		//v = (englisch value) Wert Dunkelstufe einfacher Dreisatz 0..100%
		*r = (*r / 100) * v;
		*g = (*g / 100) * v;
		*b = (*b / 100) * v;	 
		
		//Berechnung der Farbsttigung
		//s = (englisch saturation) Farbsttigung
		
		s = 100 - s; //Kehrwert berechnen
		
		max_v = (255/100) * v; //Maximal Wert berechnen
		
		diff = max_v - *r;
		*r = *r + (diff * (0.01 * s)); // x / 100 * s == x * 0.01 * s
		
		diff = max_v - *g;
		*g = *g + (diff * (0.01 * s));
		
		diff = max_v - *b;
		*b = *b + (diff * (0.01 * s)); 	
}

//############################################################################
//Hauptprogramm
int main(void)
//############################################################################
{
	unsigned char r, g, b,lock;
	unsigned int winkel;
	
	DDRD|= (1<<PD6);
		
	for (unsigned a = 0;a<MAX_WS2811_LEDS;a++)
	{	
		//Berechnung des Winkels fr die Anzahl der LEDs
		winkel = (360/MAX_WS2811_LEDS)*a;
		
		//HSV to RGB
		hsv_to_rgb(winkel,100,100,&r,&g,&b);
		
		led[a].r= r;
		led[a].g= g;
		led[a].b= b;
	}
	
	while(1)
	{	
		cli();
		ws2812_sendarray((uint8_t *)&led[0],48);		// Send array to WS2812 LED. The output pin is low after send.
		sei();
	
		_delay_ms(10);		
	}
}

